package org.kantega.jexmec.pico;

/*
 * Copyright 2009 Kantega AS
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import org.kantega.jexmec.*;
import org.picocontainer.DefaultPicoContainer;
import org.picocontainer.MutablePicoContainer;

import java.util.Collections;
import java.util.List;

/**
 * PluginLoader implementation based on PicoContainer
 */
public class PicoPluginLoader<P extends Plugin> implements PluginLoader<P> {


    private ClassLocator<Class<P>> classLocator;


    public PicoPluginLoader(ClassLocator<Class<P>> classLocator) {
        this.classLocator = classLocator;
    }

    public PicoPluginLoader() {
        this.classLocator = null;
    }


    public List<P> loadPlugins(Class<P> pluginClass, ClassLoader classLoader, ServiceLocator serviceLocator) {
        ClassLocator<Class<P>> classLocator = this.classLocator != null ? this.classLocator : new DefaultClassLocator<Class<P>>(pluginClass, "pico.txt");

        final List<Class<P>> pluginClasses = classLocator.locateClasses(classLoader);

        if(pluginClasses.size() > 0) {
            MutablePicoContainer pico = new DefaultPicoContainer();

            for(Class<P> thePluginClass : pluginClasses) {
                pico.addComponent(thePluginClass, thePluginClass);
            }

            for(ServiceKey serviceKey : serviceLocator.getServiceKeys()) {
                @SuppressWarnings("unchecked")
                Object implementation = serviceLocator.lookupService(serviceKey.getServiceClass());
                pico.addComponent(serviceKey, implementation);
            }


            return pico.getComponents(pluginClass);
        } else {
            return Collections.emptyList();
        }

    }

    public void unloadPlugins(ClassLoader classLoader) {
    }

    public void start() {

    }

    public void stop() {

    }
}
